package com.pdaaphm.biz.mapper;

import java.util.List;
import java.util.Map;

import com.pdaaphm.biz.domain.AlgorithmIoField;
import com.pdaaphm.biz.dto.SubAlgorithmDTO;
import org.apache.ibatis.annotations.Param;

/**
 * 算法输入输出字段Mapper接口
 *
 * @author xlk
 * @date 2023-07-26
 */
public interface AlgorithmIoFieldMapper
{
    /**
     * 查询算法输入输出字段
     *
     * @param id 算法输入输出字段主键
     * @return 算法输入输出字段
     */
    public AlgorithmIoField selectAlgorithmIoFieldById(Long id);

    /**
     * 查询算法输入输出字段列表
     *
     * @param algorithmIoField 算法输入输出字段
     * @return 算法输入输出字段集合
     */
    public List<AlgorithmIoField> selectAlgorithmIoFieldList(AlgorithmIoField algorithmIoField);

    /**
     * 新增算法输入输出字段
     *
     * @param algorithmIoField 算法输入输出字段
     * @return 结果
     */
    public int insertAlgorithmIoField(AlgorithmIoField algorithmIoField);

    /**
     * 修改算法输入输出字段
     *
     * @param algorithmIoField 算法输入输出字段
     * @return 结果
     */
    public int updateAlgorithmIoField(AlgorithmIoField algorithmIoField);

    /**
     * 删除算法输入输出字段
     *
     * @param id 算法输入输出字段主键
     * @return 结果
     */
    public int deleteAlgorithmIoFieldById(Long id);

    /**
     * 批量删除算法输入输出字段
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteAlgorithmIoFieldByIds(Long[] ids);

    public List<SubAlgorithmDTO> getIoSubList(@Param("subTypeId") Long subTypeId,@Param("algoId") Long algoId);

    /**
     * 根据算法id查询MatlabIoPath，Type和Index，以构建matlab完整绝对路径
     * @param id
     * @return
     */
    public List<Map> selectMatlabIoPathTypeIndexByAlgoId(Long id);

    public int deleteAlgorithmIoFieldBySubTypeId(Long id);
}
