package com.pdaaphm.biz.service.impl;

import java.util.List;
import com.pdaaphm.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.pdaaphm.biz.mapper.SubAlgorithmMapper;
import com.pdaaphm.biz.domain.SubAlgorithm;
import com.pdaaphm.biz.service.ISubAlgorithmService;

/**
 * 子算法Service业务层处理
 * 
 * @author xlk
 * @date 2023-07-26
 */
@Service
public class SubAlgorithmServiceImpl implements ISubAlgorithmService 
{
    @Autowired
    private SubAlgorithmMapper subAlgorithmMapper;

    /**
     * 查询子算法
     * 
     * @param id 子算法主键
     * @return 子算法
     */
    @Override
    public SubAlgorithm selectSubAlgorithmById(Long id)
    {
        return subAlgorithmMapper.selectSubAlgorithmById(id);
    }

    /**
     * 查询子算法列表
     * 
     * @param subAlgorithm 子算法
     * @return 子算法
     */
    @Override
    public List<SubAlgorithm> selectSubAlgorithmList(SubAlgorithm subAlgorithm)
    {
        return subAlgorithmMapper.selectSubAlgorithmList(subAlgorithm);
    }

    /**
     * 新增子算法
     * 
     * @param subAlgorithm 子算法
     * @return 结果
     */
    @Override
    public int insertSubAlgorithm(SubAlgorithm subAlgorithm)
    {
        subAlgorithm.setCreateTime(DateUtils.getNowDate());
        return subAlgorithmMapper.insertSubAlgorithm(subAlgorithm);
    }

    /**
     * 修改子算法
     * 
     * @param subAlgorithm 子算法
     * @return 结果
     */
    @Override
    public int updateSubAlgorithm(SubAlgorithm subAlgorithm)
    {
        subAlgorithm.setUpdateTime(DateUtils.getNowDate());
        return subAlgorithmMapper.updateSubAlgorithm(subAlgorithm);
    }

    /**
     * 批量删除子算法
     * 
     * @param ids 需要删除的子算法主键
     * @return 结果
     */
    @Override
    public int deleteSubAlgorithmByIds(Long[] ids)
    {
        return subAlgorithmMapper.deleteSubAlgorithmByIds(ids);
    }

    /**
     * 删除子算法信息
     * 
     * @param id 子算法主键
     * @return 结果
     */
    @Override
    public int deleteSubAlgorithmById(Long id)
    {
        return subAlgorithmMapper.deleteSubAlgorithmById(id);
    }
}
